use leptos::*;
use leptos_router::*;
use serde::{Deserialize, Serialize};
#[cfg(feature = "ssr")]
use sqlx::FromRow;
use uuid::Uuid;

pub mod add;
pub mod view;

#[component]
pub fn SeriesList() -> impl IntoView {
    let load = create_resource(|| (), |_| async move { get_all().await });
    view! {
        <Suspense fallback=|| view!{ <div>"Loading..."</div> }>
            <ul class="list-group mb-3">
                {load().map(format_series)}
            </ul>
        </Suspense>
        <div>
            <A href="add"
                attr:class="btn btn-primary"
            >
                Add Series
            </A>
        </div>
    }
}

fn format_series(data: Result<Vec<Series>, ServerFnError>) -> impl IntoView {
    data.unwrap_or_default()
        .into_iter()
        .map(|d| {
            view! {
                <li class="list-group-item">
                    <A href=format!("view/{}", d.id)
                        attr:class="btn btn-sm btn-secondary"
                        attr:style="float: right"
                    >
                        View Series
                    </A>
                    {d.name}
                </li>
            }
        })
        .collect_view()
}

#[derive(Clone, Debug, PartialEq, Eq, Serialize, Deserialize)]
#[cfg_attr(feature = "ssr", derive(FromRow))]
pub struct Series {
    id: Uuid,
    name: String,
}

#[server]
pub async fn get_all() -> Result<Vec<Series>, ServerFnError> {
    use sqlx::{Pool, Postgres};
    use tokio_stream::StreamExt;

    let pool: Pool<Postgres> = expect_context();

    let mut r = sqlx::query_as::<_, Series>("select * from series;").fetch(&pool);

    let mut found = Vec::new();
    while let Some(series) = r.try_next().await? {
        found.push(series);
    }

    Ok(found)
}
